#include "api/df_setup_records.h"
#include "datafox_library_iv.h"

DF_API_EXTERN_DECORATE int DF_API_CALL DFCLoadDatensatzbeschreibung( int connectionNumber, int deviceAddress, int *errorNumber )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionNumber );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        if ( errorNumber != nullptr )
        {
            *errorNumber = IV_ERROR_CHANNEL;
        }

        return 0;
    }

    channelBridge->logBegin( __func__, deviceAddress, "" );

    // Gerteadresse auf lokale mappen.
    uint8_t l_device_address = channelBridge->cast_device_address( deviceAddress );

    // zulaessige Bereiche pruefen
    if ( l_device_address == 0 )
    {
        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_PARAMETER, errorNumber );
        channelBridge->logEnd( "\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    // Schnittstelle geoeffnet?
    if ( channelBridge->isOpen() == false )
    {
        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_CHANNEL_CLOSED, errorNumber );
        channelBridge->logEnd( "\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_SUCCESS, errorNumber );

    int l_error_number = IV_ERROR_SUCCESS;
    int iRes = channelBridge->getTableDescription( deviceAddress, 0, &l_error_number );
    // Nach Kommunikationsfehler, Kanal bereinigen und ggf. Fehlernummer uebergeben.
    if ( iRes == 0 )
    {
        channelBridge->setLastErrorNumber( deviceAddress, l_error_number, errorNumber );
        channelBridge->resetWithError( channelBridge->getLastErrorNumber( deviceAddress ) );
    }

    channelBridge->logEnd( "\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, iRes, channelBridge->getLastErrorNumber( deviceAddress ) );

    return iRes;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCDatBCnt( int connectionNumber )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionNumber );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        return 0;
    }

    channelBridge->logBegin( __func__, "" );

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( 0, IV_ERROR_SUCCESS );

    int iRes = channelBridge->GetTableCount( 0 );

    channelBridge->logEnd( "\t(%i, %i)", iRes, channelBridge->getLastErrorNumber( 0 ) );

    return iRes;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCDatBDatensatz( int connectionNumber, int number, char *name, int *fieldCount )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionNumber );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        return 0;
    }

    channelBridge->logBegin( __func__, "\t%i", number );

    // zulaessige Bereiche pruefen
    if ( ((number >= 0 && number < MAX_RECORD_DESCRIPTION) == false && (number >= 100 && number <= 103) == false && (number >= 200 && number <= 207) == false)
         || name == nullptr || fieldCount == nullptr )
    {
        channelBridge->setLastErrorNumber( 0, IV_ERROR_FUNCTION_PARAMETER );
        channelBridge->logEnd( "\t\t\t(%i, %i)", 0, channelBridge->getLastErrorNumber( 0 ) );
        return 0;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( 0, IV_ERROR_SUCCESS );

    std::string szTableName;
    unsigned char ucFieldCount;

    int iRes = channelBridge->GetTableRecord( number, szTableName, &ucFieldCount );

    if ( iRes )
    {
        strcpy( name, szTableName.c_str() );
        *fieldCount = ucFieldCount;
    }

    channelBridge->logEnd( "\t%s\t%i\t(%i, %i)", name, *fieldCount, iRes, channelBridge->getLastErrorNumber( 0 ) );
    return iRes;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCDatBFeld( int connectionNumber, int dataBNumber, int number, char *name, int *type, int *size )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionNumber );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        return 0;
    }

    channelBridge->logBegin( __func__, "\t%i\t%i", dataBNumber, number );

    // zulaessige Bereiche pruefen
    if ( (dataBNumber >= 0 && dataBNumber < MAX_RECORD_DESCRIPTION) == false
         || (number >= 0 && number < MAX_FIELDS) == false
         || name == nullptr || type == nullptr || size == nullptr )
    {
        channelBridge->setLastErrorNumber( 0, IV_ERROR_FUNCTION_PARAMETER );
        channelBridge->logEnd( "\t\t\t\t(%i, %i)", 0, channelBridge->getLastErrorNumber( 0 ) );
        return 0;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( 0, IV_ERROR_SUCCESS );

    std::string szFieldName;
    unsigned char ucType, ucLength;

    int iRes = channelBridge->GetTableField( dataBNumber, number, szFieldName,
                                             &ucType, &ucLength, 0 );

    if ( iRes )
    {
        strcpy( name, szFieldName.c_str() );
        *type = ucType;
        *size = ucLength;
    }

    channelBridge->logEnd( "\t%s\t%i\t%i\t(%i, %i)", name, *type, *size, iRes, channelBridge->getLastErrorNumber( 0 ) );

    return iRes;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCComCollectData( int connectionNumber, int deviceAddress, int *errorNumber )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionNumber );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        if ( errorNumber != nullptr )
        {
            *errorNumber = IV_ERROR_CHANNEL;
        }

        return -1;
    }

    channelBridge->logBegin( __func__, deviceAddress, "" );

    // Gerteadresse auf lokale mappen.
    uint8_t l_device_address = channelBridge->cast_device_address( deviceAddress );

    // zulaessige Bereiche pruefen
    if ( l_device_address == 0 )
    {
        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_PARAMETER, errorNumber );
        channelBridge->logEnd( "\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, -1, channelBridge->getLastErrorNumber( deviceAddress ) );
        return -1;
    }

    // Schnittstelle geoeffnet?
    if ( channelBridge->isOpen() == false )
    {
        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_CHANNEL_CLOSED, errorNumber );
        channelBridge->logEnd( "\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, -1, channelBridge->getLastErrorNumber( deviceAddress ) );
        return -1;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_SUCCESS, errorNumber );

    int l_error_number = IV_ERROR_SUCCESS;
    int iRes = channelBridge->collectData( deviceAddress, &l_error_number );
    // Nach Kommunikationsfehler, Kanal bereinigen und ggf. Fehlernummer uebergeben.
    if ( iRes == -1 )
    {
        channelBridge->setLastErrorNumber( deviceAddress, l_error_number, errorNumber );
        channelBridge->resetWithError( channelBridge->getLastErrorNumber( deviceAddress ) );
    }

    channelBridge->logEnd( "\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, iRes, channelBridge->getLastErrorNumber( deviceAddress ) );
    return iRes;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCComGetDatensatz( int connectionNumber, unsigned char *buffer, int *bufferLength, int *errorNumber )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionNumber );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        if ( errorNumber != nullptr )
        {
            *errorNumber = IV_ERROR_CHANNEL;
        }

        return -1;
    }

    channelBridge->logBegin( __func__, "" );

    // zulaessige Bereiche pruefen
    if ( buffer == nullptr || bufferLength == nullptr )
    {
        channelBridge->setLastErrorNumber( 0, IV_ERROR_FUNCTION_PARAMETER, errorNumber );
        channelBridge->logEnd( "\t\t\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, -1, channelBridge->getLastErrorNumber( 0 ) );
        return -1;
    }

    // Schnittstelle geoeffnet?
    if ( channelBridge->isOpen() == false )
    {
        channelBridge->setLastErrorNumber( 0, IV_ERROR_CHANNEL_CLOSED, errorNumber );
        channelBridge->logEnd( "\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, -1, channelBridge->getLastErrorNumber( 0 ) );
        return -1;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( 0, IV_ERROR_SUCCESS, errorNumber );

    int l_error_number = IV_ERROR_SUCCESS;
    uint16_t l_bufferLength = (*bufferLength > 0xFFFF) ? 0xFFFF : static_cast<uint16_t>( *bufferLength );
    int iRes = channelBridge->getDatensatz( buffer, l_bufferLength, l_error_number );
    // Nach Kommunikationsfehler, Kanal bereinigen und ggf. Fehlernummer uebergeben.
    if ( iRes == -1 )
    {
        channelBridge->setLastErrorNumber( 0, l_error_number, errorNumber );
        channelBridge->resetWithError( channelBridge->getLastErrorNumber( 0 ) );
    }
    else
    {
        *bufferLength = l_bufferLength;
    }

    switch ( iRes )
    {
        case 1:
        case 2:
        {
            // Daten in Puffer uebergeben
            unsigned char ucTmp[IV_RECORD_MAX * 3 + 1];
            int i = 0;
            if ( channelBridge->logEnabled() )
            {
                for ( ; i < *bufferLength && i < static_cast<int>(sizeof( ucTmp ) - 3); ++i )
                {
                    sprintf( (char *)&ucTmp[i * 3], "%02X ", (unsigned char)buffer[i] );
                }
            }
            ucTmp[i * 3] = 0;

            channelBridge->logContinue( "\t%s\t%i", ucTmp, *bufferLength );
            break;
        }

        default:
            channelBridge->logContinue( "\t%s\t%i", "", 0 );
    }

    channelBridge->logEnd( "\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, iRes, channelBridge->getLastErrorNumber( 0 ) );

    return iRes;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCReadRecord( int connectionNumber, int deviceAddress, unsigned char *buffer, int *bufferLength, int *errorNumber )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionNumber );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        if ( errorNumber != nullptr )
        {
            *errorNumber = IV_ERROR_CHANNEL;
        }

        return -1;
    }

    channelBridge->logBegin( __func__, deviceAddress, "" );

    // Gerteadresse auf lokale mappen.
    uint8_t l_device_address = channelBridge->cast_device_address( deviceAddress );

    // zulaessige Bereiche pruefen
    if ( l_device_address == 0 || buffer == nullptr || bufferLength == nullptr )
    {
        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_PARAMETER, errorNumber );
        channelBridge->logEnd( "\t\t\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, -1, channelBridge->getLastErrorNumber( deviceAddress ) );
        return -1;
    }

    // Schnittstelle geoeffnet?
    if ( channelBridge->isOpen() == false )
    {
        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_CHANNEL_CLOSED, errorNumber );
        channelBridge->logEnd( "\t\t\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, -1, channelBridge->getLastErrorNumber( deviceAddress ) );
        return -1;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_SUCCESS, errorNumber );

    int l_error_number = IV_ERROR_SUCCESS;
    uint16_t l_bufferLength = (*bufferLength > 0xFFFF) ? 0xFFFF : static_cast<uint16_t>(*bufferLength);
    int res = channelBridge->readRecord( deviceAddress, buffer, l_bufferLength, l_error_number );
    // Nach Kommunikationsfehler, Kanal bereinigen und ggf. Fehlernummer uebergeben.
    if ( res == -1 )
    {
        channelBridge->setLastErrorNumber( deviceAddress, l_error_number, errorNumber );
        channelBridge->resetWithError( channelBridge->getLastErrorNumber( deviceAddress ) );

        channelBridge->logContinue( "\t\t" );
    }
    else
    {
        *bufferLength = l_bufferLength;

        // Daten fuer Log in Puffer uebergeben
        unsigned char ucTmp[IV_RECORD_MAX * 3 + 1];
        int i = 0;
        if ( res && channelBridge->logEnabled() )
        {
            for ( i = 0; i < *bufferLength && i < static_cast<int>(sizeof( ucTmp ) - 3); ++i )
            {
                sprintf( (char *)&ucTmp[i * 3], "%02X ", (unsigned char)buffer[i] );
            }
        }
        ucTmp[i * 3] = 0;

        channelBridge->logContinue( "\t%s\t%i", ucTmp, *bufferLength );
    }

    channelBridge->logEnd( "\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, res, channelBridge->getLastErrorNumber( deviceAddress ) );

    return res;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCQuitRecord( int connectionNumber, int deviceAddress, int *errorNumber )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionNumber );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        if ( errorNumber != nullptr )
        {
            *errorNumber = IV_ERROR_CHANNEL;
        }

        return -1;
    }

    channelBridge->logBegin( __func__, deviceAddress, "" );

    // Gerteadresse auf lokale mappen.
    uint8_t l_device_address = channelBridge->cast_device_address( deviceAddress );

    // zulaessige Bereiche pruefen
    if ( l_device_address == 0 )
    {
        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_PARAMETER, errorNumber );
        channelBridge->logEnd( "\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, -1, channelBridge->getLastErrorNumber( deviceAddress ) );
        return -1;
    }

    // Schnittstelle geoeffnet?
    if ( channelBridge->isOpen() == false )
    {
        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_CHANNEL_CLOSED, errorNumber );
        channelBridge->logEnd( "\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, -1, channelBridge->getLastErrorNumber( deviceAddress ) );
        return -1;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_SUCCESS, errorNumber );

    std::string param;
    int l_error_number = IV_ERROR_SUCCESS;
    int res = channelBridge->quitRecord( deviceAddress, param, &l_error_number );
    // Nach Kommunikationsfehler, Kanal bereinigen und ggf. Fehlernummer uebergeben.
    if ( res == -1 )
    {
        channelBridge->setLastErrorNumber( deviceAddress, l_error_number, errorNumber );
        channelBridge->resetWithError( channelBridge->getLastErrorNumber( deviceAddress ) );
    }

    channelBridge->logEnd( "\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, res, channelBridge->getLastErrorNumber( deviceAddress ) );

    return res;
}

DF_API_EXTERN int DF_API_CALL DFCQuitRecordExt( int connectionNumber, int deviceAddress, char *param, int *paramLength )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionNumber );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        return -1;
    }

    channelBridge->logBegin( __func__, deviceAddress, "\t%s\t%d", (param == nullptr) ? "" : param, (paramLength == nullptr) ? 0 : *paramLength );

    // Gerteadresse auf lokale mappen.
    uint8_t l_device_address = channelBridge->cast_device_address( deviceAddress );

    // zulaessige Bereiche pruefen
    if ( l_device_address == 0 )
    {
        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_PARAMETER );
        channelBridge->logEnd( "\t(%i, %i)", -1, channelBridge->getLastErrorNumber( deviceAddress ) );
        return -1;
    }

    // Schnittstelle geoeffnet?
    if ( channelBridge->isOpen() == false )
    {
        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_CHANNEL_CLOSED );
        channelBridge->logEnd( "\t(%i, %i)", -1, channelBridge->getLastErrorNumber( deviceAddress ) );
        return -1;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_SUCCESS );

    std::string par( param );
    int l_error_number = IV_ERROR_SUCCESS;
    int res = channelBridge->quitRecord( deviceAddress, par, &l_error_number );
    // Nach Kommunikationsfehler, Kanal bereinigen und ggf. Fehlernummer uebergeben.
    if ( res == -1 )
    {
        channelBridge->setLastErrorNumber( deviceAddress, l_error_number );
        channelBridge->resetWithError( channelBridge->getLastErrorNumber( deviceAddress ) );
    }

    channelBridge->logEnd( "\t(%i, %i)", res, channelBridge->getLastErrorNumber( deviceAddress ) );

    return res;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCComClearData( int connectionNumber, int deviceAddress )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionNumber );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        return 0;
    }

    channelBridge->logBegin( __func__, deviceAddress, "" );

    // Gerteadresse auf lokale mappen.
    uint8_t l_device_address = channelBridge->cast_device_address( deviceAddress );

    // zulaessige Bereiche pruefen
    if ( l_device_address == 0 )
    {
        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_PARAMETER );
        channelBridge->logEnd( "\t(%i, %i)", 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    // Schnittstelle geoeffnet?
    if ( channelBridge->isOpen() == false )
    {
        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_CHANNEL_CLOSED );
        channelBridge->logEnd( "\t(%i, %i)", 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_SUCCESS );

    int l_error_number = IV_ERROR_SUCCESS;
    int iRes = channelBridge->clearData( deviceAddress, &l_error_number );
    // Nach Kommunikationsfehler, Kanal bereinigen und ggf. Fehlernummer uebergeben.
    if ( iRes == 0 )
    {
        channelBridge->setLastErrorNumber( deviceAddress, l_error_number );
        channelBridge->resetWithError( channelBridge->getLastErrorNumber( deviceAddress ) );
    }

    channelBridge->logEnd( "\t(%i, %i)", iRes, channelBridge->getLastErrorNumber( deviceAddress ) );

    return iRes;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCRestoreRecords( int connectionNumber, int deviceAddress, int *errorNumber )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionNumber );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        if ( errorNumber != nullptr )
        {
            *errorNumber = IV_ERROR_CHANNEL;
        }

        return 0;
    }

    channelBridge->logBegin( __func__, deviceAddress, "" );

    // Gerteadresse auf lokale mappen.
    uint8_t l_device_address = channelBridge->cast_device_address( deviceAddress );

    // zulaessige Bereiche pruefen
    if ( l_device_address == 0 )
    {
        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_PARAMETER, errorNumber );
        channelBridge->logEnd( "\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    // Schnittstelle geoeffnet?
    if ( channelBridge->isOpen() == false )
    {
        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_CHANNEL_CLOSED, errorNumber );
        channelBridge->logEnd( "\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_SUCCESS, errorNumber );

    int l_error_number = IV_ERROR_SUCCESS;
    int iRes = channelBridge->restoreRecords( deviceAddress, &l_error_number );
    // Nach Kommunikationsfehler, Kanal bereinigen und ggf. Fehlernummer uebergeben.
    if ( iRes == 0 )
    {
        channelBridge->setLastErrorNumber( deviceAddress, l_error_number, errorNumber );
        channelBridge->resetWithError( channelBridge->getLastErrorNumber( deviceAddress ) );
    }

    channelBridge->logEnd( "\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, iRes, channelBridge->getLastErrorNumber( deviceAddress ) );

    return iRes;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCCheckData( int connectionNumber, int deviceAddress, int *errorNumber )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionNumber );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        if ( errorNumber != nullptr )
        {
            *errorNumber = IV_ERROR_CHANNEL;
        }

        return -1;
    }

    channelBridge->logBegin( __func__, deviceAddress, "" );

    // Gerteadresse auf lokale mappen.
    uint8_t l_device_address = channelBridge->cast_device_address( deviceAddress );

    // zulaessige Bereiche pruefen
    if ( l_device_address == 0 )
    {
        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_PARAMETER, errorNumber );
        channelBridge->logEnd( "\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, -1, channelBridge->getLastErrorNumber( deviceAddress ) );
        return -1;
    }

    // Schnittstelle geoeffnet?
    if ( channelBridge->isOpen() == false )
    {
        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_CHANNEL_CLOSED, errorNumber );
        channelBridge->logEnd( "\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, -1, channelBridge->getLastErrorNumber( deviceAddress ) );
        return -1;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_SUCCESS, errorNumber );

    int l_error_number = IV_ERROR_SUCCESS;
    int iRes = channelBridge->checkData( deviceAddress, &l_error_number );
    // Nach Kommunikationsfehler, Kanal bereinigen und ggf. Fehlernummer uebergeben.
    if ( iRes == -1 )
    {
        channelBridge->setLastErrorNumber( deviceAddress, l_error_number, errorNumber );
        channelBridge->resetWithError( channelBridge->getLastErrorNumber( deviceAddress ) );
    }

    channelBridge->logEnd( "\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, iRes, channelBridge->getLastErrorNumber( deviceAddress ) );

    return iRes;
}
