#pragma once
#ifndef HARDWARE_INFO_H
#define HARDWARE_INFO_H

#include "file/interface/hardware_info_structs.h"
#include "file/file_hardware_info.h"
#include <cstdint>

class HardwareInfo
{
public:
    HardwareInfo();
    virtual ~HardwareInfo();

public:
    enum EnumDeviceType
    {
        PZEMasterIV,
        ZKMasterIV,
        BDEMasterIV,
        MobilMasterIV,
        MDEBoxIV,
        AEMasterIV,
        TimeboyIV,
        ExklusivLineIV,
        FlexMasterIV,
        TimeboyMobilPZE,
            // Bis hier waren die Nummern identisch mit der HardwareInfo-Vorgängerversion.
        IPCExtensionA, // Alte HW-Info nimmt Nummer fuer: EVO_2_8_Pure
        EVO_2_8_Pure,  // Alte HW-Info nimmt Nummer fuer: EVO_4_3
        EVO_4_3,       // Alte HW-Info nimmt Nummer fuer: IPCExtensionA
        IPCExtensionB,
        IPCExtensionC,
        KYO_Inloc,     // Alte HW-Info nimmt Nummer fuer: IOBox
        IOBox,         // Alte HW-Info nimmt Nummer fuer: ZKBox
        MobilBox,
        TimeboyTBDock,
        MobileTBDock,
        KYO_Oneloc,
        EVO_3_5_Universal,
        SAMS_OCL_Reader,
        EVO_3_5_Pure,
        EVO_Agera,
        KYO_Cenloc,
        Vario_10,
        Vario_12,
        Vario_15,
        Vario_17,
        IPC_EVO_12_1,
        IPC_EVO_15,
        IPC_EVO_18_5,
        IPC_EVO_24,
        EVO_Intera_2,
        EVO_4_6_FlexKey,
        EVO_5_0_Pure
    };
public:
    void reset();

    bool importStructHardwareInfo( const StructHardwareInfo &info );
    StructHardwareInfo exportStructHardwareInfo();
    StructHardwareInfo importedStructHardwareInfo();

    void setFileHardwareInfo( const FileHardwareInfo &file );
    FileHardwareInfo &fileHardwareInfo();

public:
    bool isMasterIV();
    bool isMasterIV( const StructHardwareInfo &info );


public:
    enum EnumHardwareConfigType
    {
        Mainboard,
        Module,
        ModuleWithPlaces,
        SubModule
    };

public:
    void setVersion( uint8_t version );
    uint8_t version();
    std::string deviceName(EnumDeviceType type);
    EnumDeviceType deviceType();
    EnumDeviceType deviceTypeOld();
    uint8_t mainBoardRevision();

public:
    listItemBoard collectModuleIds( const setModuleId &ids );
    // Shorties
    listItemBoard moduleRS232Communication( const setModuleRS232Communication &module = moduleRS232Communication_all );
    listItemBoard moduleRS485Communication( const setModuleRS485Communication &module = moduleRS485Communication_all );
    listItemBoard onBoardRS485Communication( const setOnBoardRS485Communication &module = onBoardRS485Communication_all );
    listItemBoard moduleUSBCommunication( const setModuleUSBCommunication &module = moduleUSBCommunication_all );
    listItemBoard moduleLANCommunication( const setModuleLANCommunication &module = moduleLANCommunication_all );
    listItemBoard onBoardLANCommunication( const setOnBoardLANCommunication &boards = onBoardLANCommunication_all );
    listItemBoard moduleWLANCommunication( const setModuleWLANCommunication &module = moduleWLANCommunication_all );
    listItemBoard moduleMobileCommunication( const setModuleMobileCommunication &module = moduleMobileCommunication_all );
    listItemBoard moduleBluetoothCommunication( const setModuleBluetoothCommunication &module = moduleBluetoothCommunication_all );
    listItemBoard moduleMMCCommunication( const setModuleMMCCommunication &module = moduleMMCCommunication_all );
    listItemBoard moduleUSBHostCommunication( const setModuleUSBHostCommunication &module = moduleUSBHostCommunication_all );

    listItemBoard moduleAccessControl( const setModuleAccessControl &module = moduleAccessControl_all );
    listItemBoard onBoardAccessControl( const setOnBoardAccessControl &boards = onBoardAccessControl_all );
    listItemBoard moduleBiokey( const setModuleBiokey &module = moduleBiokey_all );
    listItemBoard moduleGPS( const setModuleGPS &module = moduleGPS_all );

    listItemBoard moduleDisplay( const setModuleDisplay &module = moduleDisplay_all );
    listItemBoard moduleTouch( const setModuleTouch &module = moduleTouch_all );
    listItemBoard moduleTemperatureSensor( const setModuleTemperatureSensor &module = moduleTemperatureSensor_all );

private:
    uint8_t m_version;
    FileHardwareInfo m_fileHardwareInfo;
    // Uebergangsweise beim Import die zugrundeliegende Struktur merken.
    StructHardwareInfo m_hwInfo;
};

#endif
