#pragma once
#ifndef EXCHANGE_CLASSES_H
#define EXCHANGE_CLASSES_H

#include "exchange_structs.h"
#include <string>
#include <array>
#include <vector>

namespace Datafox
{
	
class FileHeader
{
public:
    typedef std::array<uint8_t, sizeof( StructFileHeader )> DataType;
    typedef std::vector<uint8_t> SerializeType;

public:
    FileHeader( EnumFileId id );
	
public:
    EnumFileId id() const;
    int size() const;
    StructFileHeader &data()
    {
        return reinterpret_cast<StructFileHeader &>( m_structData );
    }

public:
    void resetData();
    void writeSerialize( SerializeType::iterator iter );
    void readSerialize( SerializeType::const_iterator iter );
    void readSerialize( SerializeType::const_iterator iter, const SerializeType &buffer );

protected:
    EnumFileId m_id;
    DataType m_structData;
};

class StructArray
{
public:
    typedef std::array<uint8_t, sizeof( StructStructArray )> DataType;
    typedef std::vector<uint8_t> SerializeType;

public:
    StructArray();

public:
    EnumStructId id() const;
    int size() const;
    StructStructArray &data();

public:
    void resetData();
    void writeSerialize( SerializeType::iterator iter );
    void readSerialize( SerializeType::const_iterator iter );
    void readSerialize( SerializeType::const_iterator iter, const SerializeType &buffer );

protected:
    EnumStructId m_id;
    DataType m_structData;
#ifdef _DEBUG
    StructStructArray *m_data;
#endif

public:
    std::string toJson();
};

/*
template<class StructExchange> class ExchangeBase
{
public:
    typedef std::array<uint8_t, sizeof( StructExchange )> DataType;
    typedef std::vector<uint8_t> SerializeType;
    
public:
	ExchangeBase( StructId id );
    
public:
    const StructId id() const;
    const int size() const;
    StructExchange &data();

public:
    void reset();
    void writeSerialize( SerializeType::iterator iter );
    void readSerialize( SerializeType::const_iterator iter );
    void readSerialize( SerializeType::const_iterator iter, const SerializeType &buffer );

protected:
    StructId m_id;
    DataType m_structData;
};
*/
}

#endif
